/*
 * Copyright © 2012, 2020, 2022-2023 Nick Bowler
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <https://www.gnu.org/licenses/>.
 */

#ifndef CDECL_TEST_H_
#define CDECL_TEST_H_

#include <stddef.h>
#include <stdbool.h>
#include <limits.h>
#include <assert.h>

#ifndef _
#  define _(x) (x)
#endif

#define MIN(a, b) ((a) < (b) ? (a) : (b))

struct cdecl_declspec;
struct option;
struct cdecl;

struct test_rng;

void *malloc_nofail(size_t size);
void *realloc_nofail(void *ptr, size_t size);
void test_print_specifiers(struct cdecl_declspec *spec);
void test_print_decl(struct cdecl *decl);
void test_explain_decl(struct cdecl *decl);

bool strict_strtoul(unsigned long *val, const char *str, int base);

void test_print_version(const char *program);
void test_print_options(const struct option *lopts);

/*
 * Allocate a new random number generator with the given seed string (which
 * should normally be a command-line argument).  The string is parsed as an
 * integer value as if by strtoull with a base of 0.
 */
struct test_rng *test_rng_alloc(const char *seed);

/*
 * Free the random number generator rng.
 */
void test_rng_free(struct test_rng *rng);

/*
 * Return a random integer uniformly on the closed interval [0, limit-1]
 */
unsigned test_rng_uniform_int(struct test_rng *rng, unsigned limit);

/*
 * Return false or true with 50% probablility either way.
 */
static inline int test_rng_50_50(struct test_rng *rng)
{
	return test_rng_uniform_int(rng, 2) == 0;
}

#endif
